/*
 * FMATHADD is distributed under the GNU GENERAL PUBLIC LICENSE (GPL)
 * Version 2 (June 1991). See the "COPYING" file distributed with this
 * software for more info.
 *
 * Copyright (C) 2001 by Michael Schroeder <mdhs@gmx.net>
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "fmath.h"
#include "starcalc.h"
#include "locals.h"



/*
 * Polynom-Entwicklung
 */
SO_CALLBACK(SC_poly, double *r, double *x, NUMBERARRAY *array)
{
  double d = array->first.value;	/* 1. Koeffizient */
  int count = array->count;		/* Anzahl aller Zellen */
  SUBNUMBER *cell = &array->first;	/* Zeiger auf 1. Zelle */

  /* nur eindimensionale Polynome sind zugelassen */

  if (cell->error || ISDIM2(array)) {
    *r = INVALID;
    return;
  }

  while (--count) {		/* alle Zellen durchlaufen */

    cell++;			/* Zeiger auf nchste Zelle */

    if (cell->error) {		/* bei Fehlern abbrechen */
      *r = INVALID;
      return;
    }

    d = d * *x + cell->value;	/* Horner-Schema */
  }

  *r = d;
}


/*
 * Lagrange-Interpolation
 */
SO_CALLBACK(SC_lagrange, double *r, double *x, NUMBERARRAY *array)
{
  double *tab;
  int i, count = array->count;
  SUBNUMBER *cell = &array->first;	/* Zeiger auf 1. Zelle */

  /* nur Wertepaare sind zugelassen */

  if (!ISDATX2(array) || ISODD(count) || !(tab = alloca(sizeof(*tab) * count))) {
    *r = INVALID;
    return;
  }

  for (i = 0; i < count; i++, cell++) {

    if (cell->error) {		/* bei Fehlern abbrechen */
      *r = INVALID;
      return;
    }

    if (DIMCOLS(array) == 1)	/* Tabellentyp bestimmen: senkrecht? */

      tab[i] = cell->value;

    else	 		/* sonst Spalten mit Reihen tauschen */

      tab[2*i + ((2*i >= count)? 1-count: 0)] = cell->value;
      
  }

  *r = lagrange(*x, count/2, (void*)tab); 
}


/*
 * Interpolation zur Zahl mit 3, 4 oder 5 Tabellenwerten
 */
SO_CALLBACK(SC_ipfunc, double *r, double *x, NUMBERARRAY *array)
{
  double dat[5];
  int i, count = array->count;		/* Anzahl aller Zellen */
  SUBNUMBER *cell = &array->first;	/* Zeiger auf 1. Zelle */

  /* nur 3 bis 5 Werte sind zugelassen */

  if (count < 3 || count > 5) {
    *r = INVALID;
    return;
  }

  for (i = 0; i < count; i++, cell++) {		/* alle Zellen durchlaufen */

    if (cell->error) {		/* bei Fehlern abbrechen */
      *r = INVALID;
      return;
    }

    dat[i] = cell->value;
  }

  switch (count) {

    case 3:
      *r = ip3_func(*x, dat);
      return;
      
    case 4:
      *r = ip4_func(*x, dat);
      return;
      
    case 5:
      *r = ip5_func(*x, dat);
  }
}


/*
 * Inverse Interpolation zum Funktionswert mit 3, 4 oder 5 Tabellenwerten
 */
SO_CALLBACK(SC_iproot, double *r, double *x, NUMBERARRAY *array)
{
  double dat[5];
  int i, count = array->count;		/* Anzahl aller Zellen */
  SUBNUMBER *cell = &array->first;	/* Zeiger auf 1. Zelle */

  /* nur 3 bis 5 Werte sind zugelassen */

  if (count < 3 || count > 5) {
    *r = INVALID;
    return;
  }

  for (i = 0; i < count; i++, cell++) {		/* alle Zellen durchlaufen */

    if (cell->error) {		/* bei Fehlern abbrechen */
      *r = INVALID;
      return;
    }

    dat[i] = cell->value;
  }

  switch (count) {

    case 3:
      *r = ip3_root(*x, dat);
      return;
      
    case 4:
      *r = ip4_root(*x, dat);
      return;
      
    case 5:
      *r = ip5_root(*x, dat);
  }
}


/*
 * Interpolation zum Extremwert mit 3, 4 oder 5 Tabellenwerten
 */
SO_CALLBACK(SC_ipextr, double *r, NUMBERARRAY *array)
{
  double dat[5];
  int i, count = array->count;		/* Anzahl aller Zellen */
  SUBNUMBER *cell = &array->first;	/* Zeiger auf 1. Zelle */

  /* nur 3 bis 5 Werte sind zugelassen */

  if (count < 3 || count > 5) {
    *r = INVALID;
    return;
  }

  for (i = 0; i < count; i++, cell++) {		/* alle Zellen durchlaufen */

    if (cell->error) {		/* bei Fehlern abbrechen */
      *r = INVALID;
      return;
    }

    dat[i] = cell->value;
  }

  switch (count) {

    case 3:
      *r = ip3_extr(dat);
      return;
      
    case 4:
      *r = ip4_extr(dat);
      return;
      
    case 5:
      *r = ip5_extr(dat);
  }
}













